import javax.swing.JTextField;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.awt.Image;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.CyclicBarrier;
import java.util.regex.PatternSyntaxException;

public class Safe extends JPanel implements ActionListener {
	private static final long serialVersionUID = -7609637421193570838L;
	final static int WIDTH = 180, HEIGHT = 176, CENTERX = 90, CENTERY = 88;
	private Image dial, dollars, busted;
	private boolean success, failure;
	private AffineTransform rotation;
	private JFrame application;
	private JTextField input;
	private int[] secret = { 12, 11, 12 };

	private CountDownLatch right;
	private CyclicBarrier wrong;


	public Safe(String title) {
		rotation = new AffineTransform();
		application = new JFrame(title);
		input = new JTextField();
		input.addActionListener(this);
		input.setHorizontalAlignment(JTextField.CENTER);
		setPreferredSize(new Dimension(WIDTH, HEIGHT));
		dial = getToolkit().createImage("safe.jpg");
		busted = getToolkit().createImage("busted.jpg");
		dollars = getToolkit().createImage("dollars.jpg");
		application.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		new Thread(new Runnable() {

			public void run() {
				wrong = new CyclicBarrier(4);
				try {
					wrong.await( );
                                } catch (InterruptedException e) { }
				  catch (BrokenBarrierException bbe) {
					bbe.printStackTrace();
                                }
				alarm();
				wrong.reset();
			}
		}).start();

		new Thread(new Runnable() {

			public void run() {
				right = new CountDownLatch(1);
				try {
					right.await();
				} catch (InterruptedException e) { }
				open();
			}
		}).start();
	}

	public void launchFrame() {
		application.add(input, BorderLayout.NORTH);
		application.add(this, BorderLayout.CENTER);
		application.pack();
		application.setVisible(true);
	}

	public void paintComponent(Graphics context) {
		super.paintComponent(context);
		context.setColor(Color.white);
		context.fillRect(0, 0, getWidth(), getHeight());
		Graphics2D display = (Graphics2D) context;
		if (success)
			display.drawImage(dollars, 0, 0, WIDTH, HEIGHT, Color.white, this);
		else if (failure)
			display.drawImage(busted, 0, 0, WIDTH, HEIGHT, Color.white, this);
		else
			display.drawImage(dial, rotation, this);
	}

	public synchronized void actionPerformed(ActionEvent e) {
		int[] data = null;
		boolean valid = false;
		try {
			String[] tokens = e.getActionCommand().split("-");
			data = new int[tokens.length];
			for (int i = 0; i < tokens.length; i++)
				data[i] = Integer.parseInt(tokens[i]) % 360;
			valid = true;
		} catch (NumberFormatException nfe) {
			input.setText("invalid input");
		} catch (PatternSyntaxException pse) {
			input.setText("invalid input");
		}
		if (valid) {
			final int[] code = data;
			new Thread(new Runnable() {

				public void run() {
					for (int i = 0; i < code.length; i++) {
						adjust(code[i], 20).run();
						reset(code[i], 10).run();
					}

					if (code[0] != secret[0] || code[1] != secret[1]
							|| code[2] != secret[2]) {
						new Thread(new Runnable( ) {
                                                  public void run() {
                                                    try {
                                                       wrong.await();
                                                    } catch (InterruptedException e) { }
                                                      catch (BrokenBarrierException e) {
                                                      e.printStackTrace();
                                                    }
                                                  }
                                                } ).start();
					}

					if (code[0] == secret[0] && code[1] == secret[1]
							&& code[2] == secret[2]) {
						right.countDown();
					}
				}
			}).start();
		}
	}
	
	public void rotate(double angle, int speed) {
		rotation = new AffineTransform();
		rotation.rotate(angle, CENTERX, CENTERX);
		repaint();
		try {
			Thread.sleep(speed);
		} catch (InterruptedException ie) {
		}
	}

	public void alarm() {
		failure = true;
		repaint();
	}

	public void open() {
		success = true;
		repaint();
	}

	public Runnable adjust(final int degree, final int speed) {
		final double step = Math.toRadians(2), limit = Math.toRadians(degree);
		return new Runnable() {

			public void run() {
				for (double angle = 0; angle < limit; angle += step)
					rotate(angle, speed);
			}
		};

	}

	public Runnable reset(final int degree, final int speed) {
		final double step = Math.toRadians(2), limit = Math.toRadians(degree);
		return new Runnable() {

			public void run() {
				for (double angle = limit; angle > 0; angle -= step)
					rotate(angle, speed);
			}
		};
	}

	public static void main(String[] args) {
		Safe Safe = new Safe("Bank Safe");
		Safe.launchFrame();
	}
}
